require 'nokogiri'
require 'fileutils'
require 'spigit_conf/options'
require 'spigit_conf/core/ext/class'

module SpigitConf
  class Engage
    spigit_attr :ACTIVITY_FEED_THROTTLE
    spigit_attr :ACTIVITY_STREAM_SCHEMA
    spigit_attr :ACTIVITY_STREAM_SERVER_URL
    spigit_attr :BASE_DOMAIN
    spigit_attr :BROKER_HOST
    spigit_attr :BROKER_PORT
    spigit_attr :COMPUTE_CATEGORY_REPUTATION
    spigit_attr :COOKIE_DOMAIN
    spigit_attr :DB_NAMES
    spigit_attr :DB_NAME_USER
    spigit_attr :DRAG_AND_DROP_ENABLED
    spigit_attr :FORM_ID_CHECKED
    spigit_attr :GRID_BASED_THEME_ONLY
    spigit_attr :IDEAS_RECOMMENTORS
    spigit_attr :IDEAS_RECOMMENTORS_WEIGHTS
    spigit_attr :JASPERSERVER_URL
    spigit_attr :JSP_SECTOR_THREADS_HOME
    spigit_attr :LABEL_EDASHBOARD
    spigit_attr :LABEL_LEADERBOARD
    spigit_attr :MAX_IDLE_BROWSER_PERIOD
    spigit_attr :PARAM_INFO_MESSAGE
    spigit_attr :RECOMMENDED_IDEAS_WINDOW
    spigit_attr :RESOURCE_SERVER_URL
    spigit_attr :SEARCH_HOST
    spigit_attr :SEARCH_SERVER_PORT
    spigit_attr :SECURITY_LEVEL
    spigit_attr :SITEMAP_FILE_PATH
    spigit_attr :SITE_ID
    spigit_attr :SPIGIT_FROM_DISPLAY_NAME
    spigit_attr :SSO_REDIRECT_LOGIN_URL
    spigit_attr :SSO_REDIRECT_REGISTER_URL
    spigit_attr :TM_CONTENT_FIELD
    spigit_attr :TM_NUMBER_TOPIC
    spigit_attr :URI_ADDUSER_PAGE
    spigit_attr :URI_AGGREGATED_BLOGS_PAGE
    spigit_attr :URI_BLOG_MODERATE_PAGE
    spigit_attr :URI_COMMUNITY
    spigit_attr :URI_DASHBOARD
    spigit_attr :URI_DASHBOARD
    spigit_attr :URI_EDITPROFILE
    spigit_attr :URI_HOMEPAGE
    spigit_attr :URI_LEADERBOARD
    spigit_attr :URI_MARKET_HOME
    spigit_attr :URI_MOST_ACTIVE_MARKETS
    spigit_attr :URI_PAGE_POSTBLOGCOMMENT
    spigit_attr :URI_PAGE_POSTBLOGENTRY
    spigit_attr :URI_PENDING_APPROVALS_ACTION
    spigit_attr :URI_POSTIDEA
    spigit_attr :URI_POSTSECTOR
    spigit_attr :URI_POST_MARKET_PAGE
    spigit_attr :URI_PREDICATION_MARKET_HOME
    spigit_attr :URI_RECENT_MARKETS
    spigit_attr :URI_REPORTS_HOME
    spigit_attr :URI_RESETUSER
    spigit_attr :URI_SEARCH
    spigit_attr :URI_STORE
    spigit_attr :URI_STORE_HOME_PAGE
    spigit_attr :URI_TAG_SEARCH
    spigit_attr :URI_TRADE_SHARES_PAGE
    spigit_attr :URI_VIEWBLOG
    spigit_attr :URI_VIEWBLOGENTRY
    spigit_attr :URI_VIEWIDEA
    spigit_attr :URI_VIEWIDEA_VERSION
    spigit_attr :URI_VIEWPROFILE
    spigit_attr :URI_VIEWSECTOR
    spigit_attr :URI_VIEW_ALL_BLOGS_PAGE
    spigit_attr :URI_VIEW_BLOGCOMMENT

    attr_reader :doc, :file, :spigit_config

    def initialize(opts = {})
      @file = opts.fetch(:file)
      @doc = Nokogiri::XML(File.read(@file)) { |x| x.noblanks }
      @spigit_config = @doc.at_css('spigit-config')
    end

    def save!(opts = {})
      backup = opts.fetch(:backup, true)
      take_backup if backup

      File.open(file, 'w') do |f|
        f.write(doc.to_xml(save_with: SpigitConf::XML_SAVE_OPTIONS, :indent => 4, :encoding => 'UTF-8'))
      end
    end

    private

    def take_backup
      timestamp       = Time.now.strftime('%Y%m%d-%H%M%Z')
      backup_filename = [ file, timestamp].join('_')

      ext = 0
      while File.exists?(backup_filename)
        backup_filename = [ file, timestamp, ext].join('_')
        ext += 1
      end

      FileUtils.cp(file, backup_filename)
    end

    def add_config(key)
      if node_exists?(key)
        return get_node(key)
      end

      # Create the new node
      node = Nokogiri::XML::Node.new(key, doc)

      # Add the new node to the spigit-config namespace
      spigit_config.add_child(node)

      node
    end

    def get_node(key)
      spigit_config.at_xpath("./#{key}")
    end

    def get_nodes(key)
      spigit_config.xpath("./#{key}")
    end

    def node_exists?(key)
      elements = get_nodes(key)

      if elements.length == 0
        return false
      elsif elements.length == 1
        return true
      elsif element.length > 1
        remove_nodes(elements[1..-1])
        return true
      else
        return false
      end
    end

    def remove_nodes(*nodes)
      nodes.each do |node|
        node.remove
      end
    end
  end
end
